#!/bin/bash

# Copyright (c) 2020 by Pulse Secure, LLC. All rights reserved
#set -x

PLATFORM=`uname`

# All disk space in MegaBytes
CEF_DISPLAY_NAME="Chromium Embedded Browser"

if [ "$PLATFORM" == "Linux" ]; then
    TMP_SPACE_NEEDED=1276
    INSTALL_SPACE_NEEDED=1063
    CEF_INSTALL_ROOT_DIR=/opt
    CEF_INSTALL_DIR=${CEF_INSTALL_ROOT_DIR}/pulsesecure/lib/cefRuntime
    CEF_LIB_NAME=libcef.so
    CEF_URL=https://pcstp.pulsesecure.net/cef/linux/cef64.tar.bz2
    CEF_PACKAGE_SHA2=6bc1e4e248dec4723f41452f079720623398a2aa8c66a7c122d76ee54013d91b
    CEF_CHECK_INSTALL_PATH=$CEF_INSTALL_DIR/Release/$CEF_LIB_NAME
    CEF_DOWNLOAD_FILE=cef64.tar.bz2
    CEF_PACKAGE_NAME=cef_binary_80.1.14+ga33bdbc+chromium-80.0.3987.149_linux64_minimal
    CEF_RESOURCE_DIR=$CEF_PACKAGE_NAME/Resources
    CEF_RELEASE_DIR=$CEF_PACKAGE_NAME/Release
    PULSE_LOG_DIR="/var/log/pulsesecure/logging"
else
    TMP_SPACE_NEEDED=300
    INSTALL_SPACE_NEEDED=200
    CEF_INSTALL_ROOT_DIR=/Library
    CEF_DIRNAME="Chromium Embedded Framework.framework"
    CEF_INSTALL_DIR2="${CEF_INSTALL_ROOT_DIR}/Application Support/Pulse Secure"
    CEF_INSTALL_DIR="${CEF_INSTALL_DIR2}/${CEF_DIRNAME}"
    CEF_LIB_NAME="Chromium Embedded Framework"
    CEF_URL=https://pcstp.pulsesecure.net/cef/macOS/cef64.zip
    CEF_PACKAGE_SHA2=9bb48f55409dd3b9f9c9a5106589e247a72ee92df8618ca244d11db433433cfa
    CEF_CHECK_INSTALL_PATH="$CEF_INSTALL_DIR/$CEF_LIB_NAME"
    CEF_DOWNLOAD_FILE=cef64.zip
    PULSE_LOG_DIR="/var/log/Pulse Secure/Logging"
fi

TMP_DIR=/tmp/cef.download
REINSTALL_CEF=false

LOG_DIR=$PULSE_LOG_DIR
TMP_DIR_PARAM=tmpDirPath
SCRNAME=`basename $0`
attempting_copy=false
CURL=/usr/bin/curl
WGET=/usr/bin/wget

function printUsage()
{
    echo "Utility to download and install $CEF_DISPLAY_NAME."
    echo "Usage: $SCRNAME <check_installed|install|reinstall|uninstall> [OPTIONS]"
    echo ""
    echo "    $SCRNAME check_installed"
    echo "                 Checks and informs if currently installed or not."
    echo "    $SCRNAME <install|reinstall> [-$TMP_DIR_PARAM <Path>]"
    echo "                 Downloads and installs the $CEF_DISPLAY_NAME."
    echo "                 An optional -$TMP_DIR_PARAM can be passed, where download and"
    echo "                 extraction will happen. This directory will be cleared by the"
    echo "                 end of installation."
    echo "                 install will install CEF only if it is not installed."
    echo "                 reinstall will remove and reinstall if it is installed."
    echo "    $SCRNAME uninstall"
    echo "                 Uninstall CEF libraries"
    echo "    Note:"
    echo "        * The install can be run only with root privileges"
    echo "        * $TMP_SPACE_NEEDED MB of free space is needed in the"
    echo "          $TMP_DIR_PARAM. This space is used only during installation"
    echo "          time and freed by the end of installation."
    echo "        * $INSTALL_SPACE_NEEDED MB of free space is needed in the ${CEF_INSTALL_ROOT_DIR}"
    exit 1
}

function get_time()
{
    date '+%Y/%m/%d %H:%M:%S.%N'
}

function log_message()
{
    LOG_FILE="$LOG_DIR/cefinstaller.log"
    msg=$1
    echo $msg
    echo "$(get_time) $msg" >> "$LOG_FILE"
}

ALREADY_INSTALLED=false
function check_installed()
{
    if [ -f "$CEF_CHECK_INSTALL_PATH" ]; then
        log_message "$CEF_DISPLAY_NAME is installed"
        ALREADY_INSTALLED=true
    else
        log_message "$CEF_DISPLAY_NAME is not installed"
    fi
}

function get_root_dir()
{
    echo "$1" | cut -d "/" -f2
}

function get_file_system()
{
    df $1 | grep -vw Filesystem | awk {'print $1'}
}

function get_free_space()
{
    df -m $1 | grep -vw Filesystem | awk {'print $4'}
}

function get_space_util()
{
    df -m $1 | grep -vw Filesystem | awk {'print $5'}
}

function get_fs_desc()
{
    df -m $1 | grep -vw Filesystem | awk {'printf "%s(%s)", $6, $1'}
}

function get_space_message()
{
    echo "Not enough free disk space on $(get_fs_desc $1). Needed:$2MB Available:$3MB"
}

function check_partition_space()
{
    dirPath=$1
    freespaceneeded=$2
    freediskspace=$(get_free_space $dirPath)
    if [ $freediskspace -le $freespaceneeded ]; then
        log_message "$(get_space_message $dirPath $freespaceneeded $freediskspace)"
        exit 2
    fi
}

function check_root_privilege()
{
    if [ $EUID -ne 0 ]; then
        log_message "'$SCRNAME install|reinstall|uninstall' needs to be run with root privileges"
        exit 3
    fi
}

function check_disk_space()
{
    log_message "Checking disk space needed..."
    if [ ! -d "$TMP_DIR" ]; then
        mkdir -p "$TMP_DIR"
    fi
    tmpPartition=$(get_file_system $TMP_DIR)
    cefInstallRoot=/$(get_root_dir $CEF_INSTALL_DIR)
    installPartition=$(get_file_system $cefInstallRoot)
    if [ "$tmpPartition" == "$installPartition" ]; then
        check_partition_space $cefInstallRoot $(($TMP_SPACE_NEEDED + $INSTALL_SPACE_NEEDED))
    else
        check_partition_space $cefInstallRoot $INSTALL_SPACE_NEEDED
        check_partition_space $TMP_DIR $TMP_SPACE_NEEDED
    fi
    log_message "Disk space available"
}

function return_error()
{
    exitValue=$1
    errorMessage=$2

    if [ "100%" == "$(get_space_util $TMP_DIR)" ]; then
        log_message "Warning: $TMP_DIR is full"
    fi

    log_message "$errorMessage"
    if [ $exitValue -ne 0 ]; then
        cleanup
    fi
    exit $exitValue
}

function cleanup()
{
    log_message "Cleaning up..."
    if [ -d $TMP_DIR/$CEF_PACKAGE_NAME ]; then
        rm -rf "$TMP_DIR/$CEF_PACKAGE_NAME"
    fi
    if [ -f $TMP_DIR/$CEF_DOWNLOAD_FILE ]; then
        rm -rf "$TMP_DIR/$CEF_DOWNLOAD_FILE"
    fi
    if [ "$attempting_copy" == "true" ]; then
        rm -rf "$CEF_INSTALL_DIR/*"
    fi
    log_message "Cleanup done"
}

function download_cef()
{
    log_message "Downloading CEF..."

    RETRIES=3
    if [ -f $CURL ]; then
        $CURL --retry $RETRIES -C - $CEF_URL -o $TMP_DIR/$CEF_DOWNLOAD_FILE --connect-timeout 30
    elif [ -f $WGET ]; then
        $WGET --tries $RETRIES --continue --show-progress $CEF_URL --directory-prefix=$TMP_DIR --connect-timeout=30
    else
        return_error 8 "Error: curl or wget are needed for downloading."
    fi

    if [ $? -ne 0 ]; then
        return_error 4 "Error: Download failed. Please try again."
    fi
    log_message "Download successful"
}

function validate_checksum()
{
    log_message "Validating the downloaded package..."
    sha2checksum=`shasum -a 256 $TMP_DIR/$CEF_DOWNLOAD_FILE | cut -d' ' -f1`
    if [ "$sha2checksum" != "$CEF_PACKAGE_SHA2" ]; then
        return_error 5 "Error: Download package file verification failed. Please try again"
    fi
    log_message "Validation successful"
}

function extract_cef()
{
    log_message "Extracting the downloaded package..."
    if [ "$PLATFORM" == "Linux" ]; then
        tar --checkpoint=.1000 -jxf $TMP_DIR/$CEF_DOWNLOAD_FILE -C $TMP_DIR/ $CEF_RESOURCE_DIR $CEF_RELEASE_DIR
    else
        unzip "$TMP_DIR/$CEF_DOWNLOAD_FILE" -d "$TMP_DIR/"
    fi
    if [ $? -ne 0 ]; then
        return_error 6 "Error: Extraction failed. Please try again"
    fi
    echo
    log_message "Extraction successful"
}

function copy_cef()
{
    log_message "Copying the $CEF_DISPLAY_NAME files..."
    if [ -d "$CEF_INSTALL_DIR" ]; then
        rm -rf "$CEF_INSTALL_DIR/*"
    else
        if [ "$PLATFORM" == "Linux" ]; then
            mkdir -p "$CEF_INSTALL_DIR"
        fi
    fi

    attempting_copy=true
    if [ "$PLATFORM" == "Linux" ]; then
        cp -r $TMP_DIR/$CEF_RESOURCE_DIR $CEF_INSTALL_DIR/
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi

        cp -r $TMP_DIR/$CEF_RELEASE_DIR $CEF_INSTALL_DIR/
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi

        cp -r $CEF_INSTALL_DIR/Resources/* $CEF_INSTALL_DIR/Release/
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi
    else
        cp -rf "$TMP_DIR/${CEF_DIRNAME}" "${CEF_INSTALL_DIR2}"
        if [ $? -ne 0 ]; then
            return_error 7 "Error: Copying of files to $CEF_INSTALL_DIR failed"
        fi
    fi
    log_message "Copying successful"
}

function install()
{
    log_message "$CEF_DISPLAY_NAME"
    if [ ! -d "$TMP_DIR" ]; then
        mkdir -p "$TMP_DIR"
    fi

    if [ "$REINSTALL_CEF" == "false" ]; then
        check_installed
        if [ "$ALREADY_INSTALLED" == "true" ]; then
            return_error 0 "Not installing again. Please use 'reinstall' option for reinstallation."
        fi
    fi
    check_root_privilege
    if [ ! -d "$PULSE_LOG_DIR" ]; then
        mkdir -p "$PULSE_LOG_DIR"
    fi
    LOG_DIR="$PULSE_LOG_DIR"
    check_disk_space
    download_cef
    validate_checksum
    extract_cef
    copy_cef
    cleanup
    log_message "Successfully installed $CEF_DISPLAY_NAME"
}

function uninstall()
{
    check_root_privilege
    if [ -d "$CEF_INSTALL_DIR" ]; then
        rm -rf "$CEF_INSTALL_DIR"
        if [ $? -ne 0 ]; then
            return_error 8 "Error: Removal of $CEF_INSTALL_DIR failed"
        else
    	    log_message "Successfully uninstalled $CEF_DISPLAY_NAME"
        fi
    fi
}
#
# Main
#

if [ ! -d "$LOG_DIR" ] || [ $EUID -ne 0 ]; then
    LOG_DIR=/tmp
fi

if [ "$1" == "check_installed" ]; then
    check_installed
elif [ "$1" == "install" ] || [ "$1" == "reinstall" ]; then
    if [ "$2" == "-$TMP_DIR_PARAM" ]; then
        if [ "$3" != "" ]; then
            TMP_DIR=$3
        fi
    elif [ "$2" != "" ]; then
        printUsage
    fi
    if [ "$1" == "reinstall" ]; then
        REINSTALL_CEF=true
    fi
    install
elif [ "$1" == "uninstall" ]; then
    uninstall
else
    printUsage
fi
